package error;

import java.text.MessageFormat;
import java.util.function.Consumer;

/**
 * Prints error messages and potentially terminates the application.
 */
public class ErrorManager {

    public static final int CRITICAL_LEVEL = 3;

    private static final String ANSI_RESET = "\u001B[0m";
    private static final String ANSI_RED = "\u001B[31m";
    private static final String ANSI_PURPLE = "\u001B[35m";

    private Runnable systemTerminatingFunction;
    private Consumer<String> outputFunction;

    public ErrorManager(Runnable systemTerminatingFunction, Consumer<String> outputFunction) {
        this.systemTerminatingFunction = systemTerminatingFunction;
        this.outputFunction = outputFunction;
    }

    public void generateError(LispException lispException) {
        outputFunction.accept(formatMessage(lispException));

        if (isCritical(lispException))
            systemTerminatingFunction.run();
    }

    private String formatMessage(LispException lispException) {
        String color = isCritical(lispException) ? ANSI_PURPLE : ANSI_RED;

        return MessageFormat.format("{0}error: {1}{2}\n", color, lispException.getMessage(), ANSI_RESET);
    }

    private boolean isCritical(LispException lispException) {
        return lispException.getSeverity() >= CRITICAL_LEVEL;
    }

}
