package interpreter;

import java.io.*;
import java.text.MessageFormat;

import error.*;
import eval.EVAL;
import parser.LispParser;
import sexpression.SExpression;

public class LispInterpreter {

    public static final String ANSI_RESET = "\u001B[0m";
    public static final String ANSI_GREEN = "\u001B[32m";

    private LispParser parser;
    private ErrorManager errorManager;
    protected PrintStream outputStream;

    public LispInterpreter(InputStream inputStream, PrintStream outputStream, ErrorManager errorManager) {
        this.errorManager = errorManager;
        this.parser = new LispParser(inputStream, inputStream.toString());
        this.outputStream = outputStream;
    }

    public void interpret() {
        printGreeting();

        for (displayPrompt(); !parser.isEof(); displayPrompt())
            printValueOfNextSExpression();

        printFarewell();
    }

    protected void printGreeting() {}

    protected void displayPrompt() {}

    private void printValueOfNextSExpression() {
        try {
            printValueOfNextSExpressionWithException();
        } catch (LispException e) {
            erasePrompt();
            errorManager.generateError(e);
        } catch (RuntimeException e) {
            erasePrompt();
            errorManager.generateError(convertToLispException(e));
        }
    }

    private void printValueOfNextSExpressionWithException() {
        SExpression sExpression = parser.getNextSExpression();
        String result = MessageFormat.format("{0}{1}{2}", ANSI_GREEN, EVAL.eval(sExpression), ANSI_RESET);

        erasePrompt();
        outputStream.println(result);
    }

    protected void erasePrompt() {}

    private LispException convertToLispException(RuntimeException e) {
        return new LispException() {

            private static final long serialVersionUID = 1L;

            @Override
            public String getMessage() {
                return e.getMessage();
            }
        };
    }

    protected void printFarewell() {
        outputStream.println();
    }

}
